function [Inhalt] = barcode_lesen(Barcode)
%%-------------------------------------------------------------------------
%       PROGRAM:    barcode_lesen(Barcode)
%       VERSION:    1.01
%          DATE:    2015-05-02
%   DESCRIPTION:    Program reads user defiend barcodes in CODE39
%                   notation and is moderately fault tolerant.
%   
%        AUTHOR:    Steffen Schulze Middendorf
%
%         INPUT:    Barcode: This progamm reads a '.JPG' file. 
%                            Content of this image have to be a barcode 
%                            in CODE39 notation. 
%
%        OUTPUT:    Inhalt: Content of the barcode. 
%
%       EXAMPLE:    barcode_lesen('C:\Users\HSHL.JPG')
%
%    DISCLAIMER:    This program is freeware. Use at your own risk. 
%                   Provided without any warranty
%--------------------------------------------------------------------------

%% Bild einlesen und Filtern
%Bild einlesen
Dateiname=Barcode;
Barcode=imread(Barcode);
%Umwandlung in Schwarz/Weiss Bild
SWBarcode=~im2bw(Barcode);
%Filter anwenden
%Elemente kleiner 100 pixel lschen
SWBarcode = bwareaopen(SWBarcode,100,8); 

%Zerstrte Schwarze Striche wieder auffllen
Filterparameter = strel('square',9);
SWBarcode = imclose(SWBarcode,Filterparameter);

%Faben wieder umkehren und kleine  Striche entfernen
SWBarcode=~SWBarcode;
Filterparameter = strel('square',13);
SWBarcode = imclose(SWBarcode,Filterparameter);

%% Anfang und Ende des Barcodes finden
%Gre des eingelesenen Bildes feststellen
Abmasse=size(SWBarcode);
xmass=Abmasse(2);
ymass=Abmasse(1);
%Gesuchte Mae initialisieren
yEnde=0;
xEnde=0;
yStart=0;
xStart=0;
CounterSchwarzePixel=0;
Anfanggefunden=0;
%Anfang des Barcodes finden in Y(Hhe), durch zhlen der schwarzen Pixel
%von oben beginnend
 for i=1:ymass
     if Anfanggefunden==1
        break        
     end
     for k=1:xmass
         if SWBarcode(i,k)== 0
             %Counter zhlt schwarze Pixel um sicherzustellen das 
             %nicht aufgrund eines nicht gefilterten schwarzen Pixels
             %der falsche Bereich ausgeschnitten wird
             CounterSchwarzePixel=CounterSchwarzePixel+1;
         end          
         %Wenn Pixelcounter > 500 wird die Schleife abgebrochen und 
         %und der yStart Wert gesichert
         if SWBarcode(i,k)== 0 && CounterSchwarzePixel > 10000
             yStart=i;
             Anfanggefunden=1;
             break
         end          
     end
 end
%Zurcksetzen der Zhler
CounterSchwarzePixel=0;
Anfanggefunden=0;
%Ende des Barcodes finden in Y(Hhe), durch zhlen der schwarzen Pixel von
%unten beginnend
for i=ymass:-1:1 
    if Anfanggefunden==1
        break        
    end
    for k=xmass:-1:1 
        if SWBarcode(i,k)== 0
            CounterSchwarzePixel=CounterSchwarzePixel+1;
            %Counter zhlt schwarze Pixel um sicherzustellen das 
            %nicht aufgrund eines nicht gefilterten schwarzen Pixels
            %der falsche Bereich ausgeschnitten wird
        end
        %Wenn Pixelcounter > 500 wird die Schleife abgebrochen und 
        %und der yStart Wert gesichert
        if SWBarcode(i,k)== 0 && CounterSchwarzePixel > 10000
            yEnde=i;
            Anfanggefunden=1;
            break
        end   
    end     
end
%Zurcksetzen der Zhler
%Anfang des Barcodes finden in X(Lnge)
Anfanggefunden=0;
for i=1:xmass
    if Anfanggefunden==1
        break        
    end
    for k=yStart:yEnde
        if SWBarcode(k,i)== 0
            xStart=i;
            Anfanggefunden=1;
            break
        end   
    end
end
%Ende des Barcodes finden in X(Lnge)
for i=1:xmass
    for k=yStart:yEnde
        if SWBarcode(k,i)== 0 && i>xEnde
            xEnde=i;
        end   
    end     
end
%% Barcode ausschneiden auf Bereich mit Code-Inhalt unter Bercksichtung der analysierten Anfang-Ende Mae
CodeZentriert=SWBarcode(yStart:yEnde,xStart:xEnde);
AbmasseZentriert = size(CodeZentriert);
%Gre des neuen zentrierten Bildes feststellen
xmassZ=AbmasseZentriert(2);
ymassZ=AbmasseZentriert(1);
%% Breite von breiten und schmalen Elementen herausfinden
%Herausfinden wie breit (Pixel) ein breiter Strich ist
%Initialisieren der gesuchten Variablen
Breit=0;
MaximalBreit=0;
for i=1:xmassZ
    if CodeZentriert(round(ymassZ/2),i) == 1
        %Es wird so lange addiert wie durchgngig schwarze Pixel vorhanden
        %sind.
        Breit=Breit+1;
    end    
    if CodeZentriert(round(ymassZ/2),i) == 0
        %Sobald ein weier Pixel unterbricht wird anaysiert ob es sich um 
        %die lngste Folge von schwarzen Pixeln gehandelt hat, wenn ja
        %wird die Lnge in Maximalbreit gesichert.
        if Breit > MaximalBreit
            MaximalBreit=Breit;
        end
        Breit=0;
    end
end 
%Maximale Breite wird genutzt um auch die Breite des schmalen Elements 
%zu bestimmen Verhltnis 1:2 ist bekannt.
Breit=MaximalBreit;
Schmal=round(Breit/2);
%Zugeschnittenes Bild anzeigen
imshow(CodeZentriert)
%% Toleranzen fr Elemente festlegen
%Toleranzen festlegen oT = obere Toleranz uT = unterere Toleranz
%Breites Element
oTBreit = round(Breit+Breit/3);
uTBreit = round(Breit-Breit/3);
%Schmales Element
oTSchmal = round(Schmal+Schmal/3);
uTSchmal = round(Schmal-Schmal/3);
%% Code nacheinander einlesen und auswerten
%Initialisieren der Variablen
Breite=0;
Ausgabe=[];
FarbeALT=0;
Anfanggefunden=0;
%Bild "CodeZentriert" durchlaufen und nach Elementen Strich/Lcke suchen
%Anfang finden(wichtig fr schrge codes)
for i=1:xmassZ 
    Farbe=CodeZentriert((round(ymassZ/2)),i);
    if Anfanggefunden ~= 1 && Farbe==0
        Anfanggefunden=1;
    end
    %Solange kein Farbwechsel stattfindet, Breiten aufaddieren
    if Anfanggefunden == 1 
        if Farbe==FarbeALT
           Breite=Breite+1;
        end
        %Wenn Farbwechsel stattfindet, vergleich um welches Element es 
        %sich handelt unter Bercksichtung der Toleranzen
        %Schmaler Strich
        if Farbe ~= FarbeALT || i==xmassZ
            if (Breite>uTSchmal) && (Breite<oTSchmal) && (FarbeALT==1)
             %Ausgabe speichert ein Array mit den Zahlen 1,2,3,4 diese
             %stehen jeweils fr ein Element
             Ausgabe=[Ausgabe 1]; 
            end
            %Schmaler Strich
            if (Breite>uTSchmal) && (Breite<oTSchmal) && (FarbeALT==0)
             Ausgabe=[Ausgabe 2]; 
            end
            %Breite Lcke
            if (Breite>uTBreit) && (Breite<oTBreit) && (FarbeALT==1)
             Ausgabe=[Ausgabe 3]; 
            end
            %Breiter Strich
            if (Breite>uTBreit) && (Breite<oTBreit) && (FarbeALT==0)
             Ausgabe=[Ausgabe 4];
            end
            Breite=1;
            FarbeALT=Farbe;
        end
    end  
end
k=0;
q=0;
Loesung=[];
Inhalt=[];
while numel(Ausgabe) > 0
   if numel(Ausgabe) > 10
        for k=1:10
            %Immer 10 Zahlen(1,2,3,4) ergeben einen Buchstaben, daher
            %werden jetzt immer 10 aufeinander folgenden Zahlen zu einem 
            %Lsungspaket zusammengechnrt
            Loesung=[Loesung Ausgabe(k)];
        end
        for k=1:10
            %Verwertete 10 Element lschen
            Ausgabe(1)=[];
        end
        %Ausgabearray wird in den wahren Inhalt transformiert. Kombination
        %von 1,2,3,4 ergbit wieder ein Zeichen, dafr wird die Funktion
        %entschlsseln genutzt
        [Zeichen] = entschluesseln(Loesung);
        %Zeichen wird in Inhalt(Ausgabearray) geschrieben
        Inhalt=[Inhalt Zeichen];
        %Lsungsarray wieder leer setzen
        Loesung=[];
   else
       %Wenn sich nur noch 10 Elemente im Ausgabe befindet wird noch einmal
       %der Inhalt von Ausgabe um 1(schmale Lcke) erweitert.
       Loesung=[Ausgabe 1];
       [Zeichen] = entschluesseln(Loesung);
       Inhalt=[Inhalt Zeichen];
       Ausgabe=[];
       Ausgabetext=['Reading and analysing of ',Dateiname,' completed succesfully. Content:']
       disp('-----------------------------------------------------------')
       disp(Inhalt)
       disp('-----------------------------------------------------------')
   end
end
end
